# -*- coding: utf-8 -*-
from PyQt4.QtCore import *
from PyQt4.QtGui import *



# Qt example layout
    
class FlowLayout(QLayout):
    def __init__(self, parent=None, margin=0, spacing=-1):
        QLayout.__init__(self, parent)
        if parent is not None:
            self.setMargin(margin)
        self.setSpacing(spacing)
        self.itemList = []


    def calc_full_width(self):
        w = 0
        for item in self.itemList:
            w += item.sizeHint().width()
        return w



    def addItem(self, item):
        self.itemList.append(item)


    def count(self):
        return len(self.itemList)


    def itemAt(self, index):
        if index >= 0 and index < len(self.itemList):
            return self.itemList[index]


    def takeAt(self, index):
        if index >= 0 and index < len(self.itemList):
            return self.itemList.pop(index)


    def expandingDirections(self):
        return Qt.Orientations(Qt.Orientation(0))


    def hasHeightForWidth(self):
        return True


    def heightForWidth(self, width):
        height = self.doLayout(QRect(0, 0, width, 0), True)
        return height


    def setGeometry(self, rect):
        QLayout.setGeometry(self, rect)
        self.doLayout(rect, False)


    def sizeHint(self):
        return self.minimumSize()


    def minimumSize(self):
        size = QSize()
        for item in self.itemList:
            size = size.expandedTo(item.minimumSize())
        size += QSize(2 * self.margin(), 2 * self.margin())
        return size


    def doLayout(self, rect, testOnly):
        x = rect.x()
        y = rect.y()
        lineHeight = 0
        for item in self.itemList:
            nextX = x + item.sizeHint().width() + self.spacing()
            if nextX - self.spacing() > rect.right() and lineHeight > 0:
                x = rect.x()
                y = y + lineHeight + self.spacing()
                nextX = x + item.sizeHint().width() + self.spacing()
                lineHeight = 0
            if not testOnly:
                item.setGeometry(QRect(QPoint(x, y), item.sizeHint()))
            x = nextX
            lineHeight = max(lineHeight, item.sizeHint().height())
        return y + lineHeight - rect.y()






